package ru.dbotthepony.mc.otm.block.entity;

import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.util.LazyOptional;
import ru.dbotthepony.mc.otm.capability.IMatterGridCell;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.matter.MatterGrid;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.block.BlockMatterCable;
import ru.dbotthepony.mc.otm.capability.IMatterHandler;
import ru.dbotthepony.mc.otm.capability.IPatternStorage;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

public class BlockEntityMatterCable extends BlockEntity implements IMatterGridCell {
	public BlockEntityMatterCable(BlockPos p_155229_, BlockState p_155230_) {
		super(Registry.BlockEntities.MATTER_CABLE, p_155229_, p_155230_);
		// OverdriveThatMatters.LOGGER.debug("create cable block entity {} {} {}", this, p_155229_, p_155230_);
	}

	private MatterGrid grid;

	private final LazyOptional<IMatterGridCell> resolver_grid = LazyOptional.of(() -> this);
	private boolean valid = true;

	@Override
	public void invalidateCaps() {
		super.invalidateCaps();
		valid = false;
	}

	@Override
	public void reviveCaps() {
		super.reviveCaps();
		valid = true;
	}

	@Nonnull
	@Override
	public <T> LazyOptional<T> getCapability(@Nonnull Capability<T> cap, @Nullable Direction side) {
		if (valid && cap == MatteryCapability.MATTER_CELL)
			return resolver_grid.cast();

		return super.getCapability(cap, side);
	}

	@Override
	public void setLevel(Level p_155231_) {
		super.setLevel(p_155231_);

		if (grid == null)
			MatterGrid.scheduleDiscoverNeighbours(this, getBlockPos(), p_155231_);
	}

	@Nullable
	@Override
	public IPatternStorage getPatternStorage() {
		return null;
	}

	@Override
	public void onNeighbourMatterCell(BlockPos pos, Level level, Direction direction, IMatterGridCell cell) {
		// OverdriveThatMatters.LOGGER.debug("Try to connect cable entity {} {} to {} {} ({})", this, pos, pos.offset(direction.getNormal()), direction, cell);
		BlockState new_state = getBlockState().setValue(BlockMatterCable.MAPPING_CONNECTION_PROP[direction.ordinal()], true);

		if (new_state != getBlockState())
			level.setBlock(getBlockPos(), new_state, Block.UPDATE_CLIENTS);
	}

	@Override
	public void setRemoved() {
		super.setRemoved();

		if (grid != null)
			grid.untrack(this);
	}

	@Nullable
	@Override
	public MatterGrid getMatterGrid() {
		return grid;
	}

	@Nullable
	@Override
	public IMatterHandler getMatterHandler() {
		return null;
	}

	@Override
	public boolean isValidMatterCell() {
		return valid;
	}

	@Override
	public void setMatterGrid(MatterGrid grid) {
		this.grid = grid;
	}
}
